{                                                                           }
{ File:       ConvertExpert.pas                                             }
{ Function:   Expert code for API Conversion Package                        }
{ Language:   Delphi for Win32                                              }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2005 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O-,P+,Q+,R+,S-,T+,U-,V+,W+,X+,Y+,Z1}
{ DEFINE Debug}
{$IFNDEF Debug}
{$D-}{$Q-}{$R-}{$S-}{$C-}{$W-}{$O+}
{$ELSE}
{$D+}{$Q+}{$R+}{$S+}{$C+}{$W+}{$O-}
{$ENDIF Debug}

unit ConvertExpert;

interface

uses
  ToolsAPI;  

procedure Register;

implementation

uses
  Classes,
  CParserData,
  ConvertUtils,
  ConvertConsts,
  ConvertExpertModule; // Data module containing actions and menu items

type
  TConvertWizard = class(TInterfacedObject, IOTAWizard, IOTANotifier,
      IOTAEditorNotifier)
  public
    // IOTANotifier
    procedure AfterSave;
    procedure BeforeSave;
    procedure Destroyed;
    procedure Modified;

    // IOTAWizard
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
    procedure Execute;

    // IOTAEditorNotifier
    procedure ViewNotification(const View: IOTAEditView; Operation:
      TOperation);
    procedure ViewActivated(const View: IOTAEditView);

    constructor Create;
    destructor Destroy; override;

  end;

procedure Register;
begin
  RegisterPackageWizard(TConvertWizard.Create);
end;

constructor TConvertWizard.Create;
var
  L: TStringList;
begin
  ConvertExpertModule.ConvertExpertData := TConvertExpertData.Create(nil);
  L := TStringList.Create;
  try
    ReadIniValues(MacrosSection, L);
    TCParserData.Macros.Sorted := True;
    TCParserData.Macros.AddStrings(L);

    L.Clear;
    ReadIniValues(TypesSection, L);
    TCParserData.Types.Sorted := True;
    TCParserData.Types.AddStrings(L);
  finally
    L.Free;
  end;
end;

destructor TConvertWizard.Destroy;
begin
  ConvertExpertModule.ConvertExpertData.Free;
end;

procedure TConvertWizard.AfterSave;
begin
  // nothing
end;

procedure TConvertWizard.BeforeSave;
begin
  // nothing
end;

procedure TConvertWizard.Destroyed;
begin
  // nothing
end;

procedure TConvertWizard.Modified;
begin
  // nothing
end;

function TConvertWizard.GetIDString: string;
begin
  Result := 'Velthuis.ConvertWizard';
end;

function TConvertWizard.GetName: string;
begin
  Result := 'ConvertWizard';
end;

function TConvertWizard.GetState: TWizardState;
begin
  Result := [];
end;

procedure TConvertWizard.Execute;
begin
  // nothing
end;

procedure TConvertWizard.ViewNotification(const View: IOTAEditView; Operation:
  TOperation);
begin
  // nothing
end;

procedure TConvertWizard.ViewActivated(const View: IOTAEditView);
begin
  // nothing
end;

end.

