{                                                                           }
{ File:       ConvertExpertModule.pas                                       }
{ Function:   DataModule with menu for API Conversion Package               }
{ Language:   Delphi for Win32                                              }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2005 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit ConvertExpertModule;

{$A8,B-,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,P+,T+,U-,V+,X+,Y+,Z1}
{ DEFINE Debug}
{$IFNDEF Debug}
  {$D-,Q-,R-,S-,C-,W-,O+}
{$ELSE}
  {$D+,Q+,R+,S+,C+,W+,O-}
{$ENDIF Debug}          

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ToolsAPI, ActnList, ConvertOptions, ImgList;

type
  TConvertExpertData = class(TDataModule)
    PresetPopupMenu: TPopupMenu;
    AddExternalSymItem: TMenuItem;
    AddHppEmitItem: TMenuItem;
    SwapDeclarationsItem: TMenuItem;
    AddJediHeaderItem: TMenuItem;
    DividerItem: TMenuItem;
    ConversionPackOptionsItem: TMenuItem;
    PresetActionList: TActionList;
    AddJediHeaderAction: TAction;
    InsertExternalSymAction: TAction;
    SwapDeclarationsAction: TAction;
    ConvertToHppEmitAction: TAction;
    ConversionPackOptionsAction: TAction;
    PresetImageList: TImageList;
    ConvertTypedefAction: TAction;
    ConvertTypedefItem: TMenuItem;
    ConvertDefineAction: TAction;
    ConvertDefineItem: TMenuItem;
    ConvertFunctionAction: TAction;
    ConvertFunctionItem: TMenuItem;
    procedure ConversionPackOptionsActionExecute(Sender: TObject);
    procedure ConvertExpertDataCreate(Sender: TObject);
    procedure ConvertExpertDataDestroy(Sender: TObject);
    procedure AddJediHeaderActionExecute(Sender: TObject);
    procedure SwapDeclarationsActionExecute(Sender: TObject);
    procedure InsertExternalSymActionExecute(Sender: TObject);
    procedure ConvertToHppEmitActionExecute(Sender: TObject);
    procedure ConvertTypedefActionExecute(Sender: TObject);
    procedure ConvertDefineActionExecute(Sender: TObject);
    procedure ConvertFunctionActionExecute(Sender: TObject);
  private
    FConversionMenu: TMenuItem;
    FSeparator: TMenuItem;
    procedure AddPresetMenuToIDE(InsertPosition: Integer; EditMenu: TMenuItem);
    procedure AddItemActionToIDE(Item: TMenuItem; Services: INTAServices);
    procedure RemoveMenuItems;
  public
  end;

var
  ConvertExpertData: TConvertExpertData;

implementation

uses
  ConvertConsts, ConvertExpertImpl; // actual implementations of actions

{$R *.DFM}

// ... TConvertExpertData ...

procedure TConvertExpertData.ConvertExpertDataCreate(Sender: TObject);
var
  EditMenu, MainMenu: TMenuItem;
  I, InsertPosition: Integer;
  Services: INTAServices;
begin
  Services := BorlandIDEServices as INTAServices;

  EditMenu := nil;

  MainMenu := Services.MainMenu.Items;
  for I := 0 to MainMenu.Count - 1 do
    if MainMenu.Items[I].Name = 'EditMenu' then
    begin
      EditMenu := MainMenu.Items[I];
      Break;
    end;

  if Assigned(EditMenu) then
  begin
    InsertPosition := -1;
    for I := 0 to EditMenu.Count - 1 do
      if EditMenu.Items[I].Name = 'EditAlignGridItem' then
      begin
        InsertPosition := I;
        Break;
      end;
    if InsertPosition = -1 then
      InsertPosition := EditMenu.Count;

    FConversionMenu := NewSubMenu(MainMenuItemCaption, 0, MainMenuItemName, []);
    EditMenu.Insert(InsertPosition, FConversionMenu);
    FSeparator := Menus.NewLine;
    EditMenu.Insert(InsertPosition + 1, FSeparator);

    AddPresetMenuToIDE(0, FConversionMenu);
  end
  else
    ShowMessage('Edit menu missing. Can not install.');
end;

procedure TConvertExpertData.ConvertExpertDataDestroy(Sender: TObject);
begin
  RemoveMenuItems;
  ConvertExpertData := nil;
end;

procedure TConvertExpertData.ConvertFunctionActionExecute(Sender: TObject);
begin
  CToDelphiConverter.ConvertFunction;
end;

procedure TConvertExpertData.ConvertTypedefActionExecute(Sender: TObject);
begin
  CToDelphiConverter.ConvertTypedef;
end;

procedure TConvertExpertData.ConvertDefineActionExecute(Sender: TObject);
begin
  CToDelphiConverter.ConvertDefine;
end;

procedure TConvertExpertData.AddJediHeaderActionExecute(Sender: TObject);
begin
  CToDelphiConverter.AddJediHeader;
end;

procedure TConvertExpertData.SwapDeclarationsActionExecute(Sender: TObject);
begin
  CToDelphiConverter.SwapDeclarations;
end;

procedure TConvertExpertData.InsertExternalSymActionExecute(Sender: TObject);
begin
  CToDelphiConverter.InsertExternalSym;
end;

procedure TConvertExpertData.ConvertToHppEmitActionExecute(Sender: TObject);
begin
  CToDelphiConverter.ConvertToHppEmit;
end;

procedure TConvertExpertData.RemoveMenuItems;
begin
  if Assigned(FConversionMenu) then
    FConversionMenu.Action.Free;
  FConversionMenu.Free;
  FSeparator.Free;
end;

procedure TConvertExpertData.AddPresetMenuToIDE(InsertPosition: Integer; EditMenu: TMenuItem);
var
  I: Integer;
  Item: TMenuItem;
  Services: INTAServices;
begin
  Services := BorlandIDEServices as INTAServices;
  for I := PresetPopupMenu.Items.Count - 1 downto 0 do
  begin
    // Remove the item from the popup menu.
    Item := PresetPopupMenu.Items[I];
    PresetPopupMenu.Items.Delete(I);
    AddItemActionToIDE(Item, Services);
    // Then add it to Delphi's menu.
    EditMenu.Insert(InsertPosition, Item);
  end;
end;

procedure TConvertExpertData.AddItemActionToIDE(Item: TMenuItem; Services: INTAServices);
var
  Action: TAction;
  ItemAction: TAction;
begin
  ItemAction := Item.Action as TAction;
  if Assigned(ItemAction) then
  begin
    Action := TAction.Create(nil);
    if ItemAction.ImageIndex >= 0 then
      Action.ImageIndex :=
        Services.ImageList.AddImage(PresetImageList, ItemAction.ImageIndex);
    Action.Caption := ItemAction.Caption;
    Action.Name := ItemAction.Name;
    Action.Hint := ItemAction.Hint;
    Action.OnExecute := ItemAction.OnExecute;
    Action.ActionList := Services.ActionList;
  end;
end;

procedure TConvertExpertData.ConversionPackOptionsActionExecute(Sender: TObject);
var
  OptionsDialog: TOptionsDialog;
begin
  OptionsDialog := TOptionsDialog.Create(nil);
  try
    OptionsDialog.ShowModal;
  finally
    OptionsDialog.Free;
  end;
end;

end.

