{                                                                           }
{ File:       ConvertOptions.pas                                            }
{ Function:   Options for conversion pack.                                  }
{ Language:   Delphi for Win32                                              }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2008,2009 Rudy Velthuis                                   }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit ConvertOptions;

{ TODO: option to convert enums to DWORD and constants. }
{ TODO: option to omit $EXTERNALSYM }
{ TODO: option to omit pretty names }
{ TODO: option to give anonymous unions names u, u2, u3, etc. }
{ TODO: set the options and implement them in the parser/lexer }
{ TODO: save and reload all options }
{ TODO: option to concatenate single lines of a function declaration, if
        they do not contain //-style comments, or turn these into brace-style
        comments. }

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Grids, StdCtrls, ComCtrls, ValEdit, ConvertListFrames,
  CParserData;

type
  TOptionsDialog = class(TForm)
    PageControl: TPageControl;
    MacrosPage: TTabSheet;
    MacrosFrame: TValueListFrame;
    TypesPage: TTabSheet;
    TypesFrame: TValueListFrame;
    OutputPage: TTabSheet;
    GeneralCaption: TLabel;
    KeepOriginalCheck: TCheckBox;
    GenerateExternalsymCheck: TCheckBox;
    GeneratePrettyNamesCheck: TCheckBox;
    TypedefsCaption: TLabel;
    ConvertToConstCheck: TCheckBox;
    ConvertToVariantCheck: TCheckBox;
    FunctionsCaption: TLabel;
    PreferVarParametersCheck: TCheckBox;
    GroupParametersCheck: TCheckBox;
    GenerateBorlandStyleCheck: TCheckBox;
    OkButton: TButton;
    CancelButton: TButton;
    HelpButton: TButton;
    GenerateDWordsCheck: TCheckBox;
    MacroFunctionsCaption: TLabel;
    GenerateInlineCheck: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure OkButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure KeepOriginalCheckClick(Sender: TObject);
    procedure GenerateExternalsymCheckClick(Sender: TObject);
    procedure GeneratePrettyNamesCheckClick(Sender: TObject);
    procedure ConvertToConstCheckClick(Sender: TObject);
    procedure ConvertToVariantCheckClick(Sender: TObject);
    procedure PreferVarParametersCheckClick(Sender: TObject);
    procedure GroupParametersCheckClick(Sender: TObject);
    procedure GenerateBorlandStyleCheckClick(Sender: TObject);
    procedure GenerateDWordsCheckClick(Sender: TObject);
    procedure GenerateInlineCheckClick(Sender: TObject);
  private
    procedure AddPresets;
    procedure WriteMacros(Strings: TStrings);
    procedure WriteTypes(Strings: TStrings);
    procedure SetOption(Option: TParserOption; Value: Boolean);
    procedure SetOptionsFromCheckBoxes;
    procedure SetCheckBoxesFromOptions;
    procedure ReadOptionsFromIni;
    procedure WriteOptionsToIni;
  public
  end;

var
  OptionsDialog: TOptionsDialog;

implementation

{$R *.dfm}

uses CParsers, CLexers, ConvertUtils, ConvertConsts, TypInfo;

procedure TOptionsDialog.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

procedure TOptionsDialog.GenerateDWordsCheckClick(Sender: TObject);
begin
  SetOption(poDWords, GenerateDWordsCheck.Checked);
end;

procedure TOptionsDialog.ConvertToConstCheckClick(Sender: TObject);
begin
  SetOption(poConstants, ConvertToConstCheck.Checked);
end;

procedure TOptionsDialog.ConvertToVariantCheckClick(Sender: TObject);
begin
  SetOption(poVariant, ConvertToVariantCheck.Checked);
end;

procedure TOptionsDialog.AddPresets;
var
  L: TStringList;
begin
  L := TStringList.Create;
  try
    ReadIniValues(MacrosSection, L);
    TCParserData.Macros.Sorted := True;
    TCParserData.Macros.AddStrings(L);
    MacrosFrame.ValueList.Strings := TCParserData.Macros;

    L.Clear;
    ReadIniValues(TypesSection, L);
    TCParserData.Types.Sorted := True;
    TCParserData.Types.AddStrings(L);
    TypesFrame.ValueList.Strings := TCParserData.Types;
  finally
    L.Free;
  end;

  ReadOptionsFromIni;
end;

procedure TOptionsDialog.FormCreate(Sender: TObject);
begin
  AddPresets;
  Application.HintHidePause := 10000;
end;

procedure TOptionsDialog.GenerateBorlandStyleCheckClick(Sender: TObject);
begin
  SetOption(poBorland, GenerateBorlandStyleCheck.Checked);
end;

procedure TOptionsDialog.GenerateExternalsymCheckClick(Sender: TObject);
begin
  SetOption(poExternalsym, GenerateExternalsymCheck.Checked);
end;

procedure TOptionsDialog.GenerateInlineCheckClick(Sender: TObject);
begin
  SetOption(poInlines, GenerateInlineCheck.Checked);
end;

procedure TOptionsDialog.GeneratePrettyNamesCheckClick(Sender: TObject);
begin
  SetOption(poPretty, GeneratePrettyNamesCheck.Checked);
end;

procedure TOptionsDialog.GroupParametersCheckClick(Sender: TObject);
begin
  SetOption(poGroup, GroupParametersCheck.Checked);
end;

procedure TOptionsDialog.OkButtonClick(Sender: TObject);
begin
  if IsPositiveResult(ModalResult) then
  begin
    WriteMacros(MacrosFrame.ValueList.Strings);
    TCParserData.Macros.Sorted := True;
    TCParserData.SetMacros(MacrosFrame.ValueList.Strings);

    WriteTypes(TypesFrame.ValueList.Strings);
    TCParserData.Types.Sorted := True;
    TCParserData.SetTypes(TypesFrame.ValueList.Strings);

    SetOptionsFromCheckBoxes;
    WriteOptionsToIni;
  end;
  Close;
end;

procedure TOptionsDialog.PreferVarParametersCheckClick(Sender: TObject);
begin
  SetOption(poVarParams, PreferVarParametersCheck.Checked);
end;

procedure TOptionsDialog.KeepOriginalCheckClick(Sender: TObject);
begin
  SetOption(poKeep, KeepOriginalCheck.Checked);
end;

procedure TOptionsDialog.WriteMacros(Strings: TStrings);
begin
  WriteIniValues(MacrosSection, Strings);
end;

procedure TOptionsDialog.WriteTypes(Strings: TStrings);
begin
  WriteIniValues(TypesSection, Strings);
end;

procedure TOptionsDialog.SetOption(Option: TParserOption; Value: Boolean);
begin
  if Value then
    TCParserData.Options := TCParserData.Options + [Option]
  else
    TCParserData.Options := TCParserData.Options - [Option];
end;

procedure TOptionsDialog.WriteOptionsToIni;
var
  I: TParserOption;
  L: TStringList;
begin
  L := TStringList.Create;
  try
    for I := Low(TParserOption) to High(TParserOption) do
      if I in TCParserData.Options then
        L.Add(GetEnumName(TypeInfo(TParserOption), Ord(I)) + '=True')
      else
        L.Add(GetEnumName(TypeInfo(TParserOption), Ord(I)) + '=False');
    WriteIniValues(OptionsSection, L);
  finally
    L.Free;
  end;
end;

procedure TOptionsDialog.ReadOptionsFromIni;
var
  L: TStringList;
  I: TParserOption;
begin
  L := TStringList.Create;
  try
    ReadIniValues(OptionsSection, L);
    for I := Low(TParserOption) to High(TParserOption) do
      SetOption(I, SameText(L.Values[GetEnumName(TypeInfo(TParserOption),
        Ord(I))], 'True'));
  finally
    L.Free;
  end;
  SetCheckBoxesFromOptions;
end;

procedure TOptionsDialog.SetOptionsFromCheckBoxes;
begin
  SetOption(poKeep, KeepOriginalCheck.Checked);
  SetOption(poExternalsym, GenerateExternalsymCheck.Checked);
  SetOption(poPretty, GeneratePrettyNamesCheck.Checked);
  SetOption(poConstants, ConvertToConstCheck.Checked);
  SetOption(poVariant, ConvertToVariantCheck.Checked);
  SetOption(poVarParams, PreferVarParametersCheck.Checked);
  SetOption(poGroup, GroupParametersCheck.Checked);
  SetOption(poBorland, GenerateBorlandStyleCheck.Checked);
  SetOption(poDWords, GenerateDWordsCheck.Checked);
  SetOption(poInlines, GenerateInlineCheck.Checked);
end;

procedure TOptionsDialog.SetCheckBoxesFromOptions;
begin
  KeepOriginalCheck.Checked := poKeep in TCParserData.Options;
  GenerateExternalsymCheck.Checked := poExternalsym in TCParserData.Options;
  GeneratePrettyNamesCheck.Checked := poPretty in TCParserData.Options;
  ConvertToConstCheck.Checked := poConstants in TCParserData.Options;
  ConvertToVariantCheck.Checked := poVariant in TCParserData.Options;
  PreferVarParametersCheck.Checked := poVarParams in TCParserData.Options;
  GroupParametersCheck.Checked := poGroup in TCParserData.Options;
  GenerateBorlandStyleCheck.Checked := poBorland in TCParserData.Options;
  GenerateDWordsCheck.Checked := poDWords in TCParserData.Options;
  GenerateInlineCheck.Checked := poInlines in TCParserData.Options;
end;

end.

