{                                                                           }
{ File:       ConvertUtils.pas                                              }
{ Function:   Utility functions for API Conversion Package                  }
{ Language:   Delphi for Win32                                              }
{ Author:     Rudy Velthuis                                                 }
{ Copyright:  (c) 2005 Rudy Velthuis                                        }
{ Disclaimer: This code is freeware. All rights are reserved.               }
{             This code is provided as is, expressly without a warranty     }
{             of any kind. You use it at your own risk.                     }
{                                                                           }
{             If you use this code, please credit me.                       }
{                                                                           }

unit ConvertUtils;

interface

uses
  Classes;

// Constructs full path of ini file.
function GetIniName: string;

// Read key/value pairs from an ini file section
procedure ReadIniValues(Section: string; Strings: TStrings);

// Read strings from an ini file section
procedure ReadIniStrings(Section: string; Strings: TStrings);

// Write key/value pairs to an ini file section
procedure WriteIniValues(Section: string; Strings: TStrings);

// Write strings to an ini file section
procedure WriteIniStrings(Section: string; Strings: TStrings);

implementation

uses
  SysUtils, IniFiles, ConvertConsts, ShlObj, Windows, Dialogs, Forms;

function GetIniName: string;
var
  AppData: array[0..MAX_PATH] of Char;
begin
  SHGetSpecialFolderPath(Application.MainFormHandle, AppData, CSIDL_APPDATA, True);
  Result := IncludeTrailingPathDelimiter(AppData) + 'Velthuis\';
  ForceDirectories(Result);
  Result := Result + IniFileName;
end;

procedure ReadIniValues(Section: string; Strings: TStrings);
var
  IniName: string;
  Ini: TIniFile;
  Sections: TStringList;
begin
  IniName := GetIniName;
  if not FileExists(IniName) then
    Exit;
  Ini := TIniFile.Create(IniName);
  try
    Sections := TStringList.Create;
    try
      Ini.ReadSections(Sections);
      if Sections.IndexOf(Section) < 0 then
        Exit;
    finally
      Sections.Free;
    end;
    Ini.ReadSectionValues(Section, Strings);
  finally
    Ini.Free;
  end;
end;

procedure ReadIniStrings(Section: string; Strings: TStrings);
var
  IniName: string;
  Ini: TIniFile;
  I: Integer;
  Sections: TStringList;
begin
  IniName := GetIniName;
  if not FileExists(IniName) then
    Exit;
  Ini := TIniFile.Create(IniName);
  try
    Sections := TStringList.Create;
    try
      Ini.ReadSections(Sections);
      if Sections.IndexOf(Section) < 0 then
        Exit;
    finally
      Sections.Free;
    end;
    Ini.ReadSection(Section, Strings);
    for I := 0 to Strings.Count - 1 do
      Strings[I] := Ini.ReadString(Section, Strings[I], '');
  finally
    Ini.Free;
  end;
end;

procedure WriteIniValues(Section: string; Strings: TStrings);
var
  Ini: TIniFile;
  IniName: string;
  I: Integer;
begin
  IniName := GetIniName;
  Ini := TIniFile.Create(IniName);
  try
    if FileExists(IniName) then
      Ini.EraseSection(Section);
    for I := 0 to Strings.Count - 1 do
      if Strings.Names[I] <> '' then
        Ini.WriteString(Section, Strings.Names[I],
          Strings.ValueFromIndex[I]);
  finally
    Ini.Free;
  end;
end;

procedure WriteIniStrings(Section: string; Strings: TStrings);
var
  Ini: TIniFile;
  IniName: string;
  I: Integer;
begin
  IniName := GetIniName;
  Ini := TIniFile.Create(IniName);
  try
    Ini.EraseSection(Section);
    for I := 0 to Strings.Count - 1 do
      Ini.WriteString(Section, 'Item' + IntToStr(I),
        Strings[I]);
  finally
    Ini.Free;
  end;
end;


end.
